@@ -1,5 +1,5 @@
 from .auth import (
-    Session
+    Session,
 )
 
 from .archive import (
@@ -8,32 +8,34 @@ from .archive import (
     Overlap,
     License,
     Band,
-    BundleOption,
-    Seat,
-    Price,
+    Bundle,
     SearchResult,
     SearchResponse,
     SearchRequest,
     OrderRequest,
-    ArchiveAPI
+    ArchiveAPI,
+    Polygon,
 )
 from .orders import (
     OrderResult,
-    OrdersAPI
+    OrdersAPI,
+    DetailedOrderResult,
 )
 
 from .arlula import (
-    ArlulaAPI
+    ArlulaAPI,
 )
 
 from .common import (
-    ArlulaObject
+    ArlulaObject,
 )
 
 from .exception import (
-    ArlulaSessionError
+    ArlulaSessionError,
+    ArlulaAPIException,
 )
 
 from .util import (
-    parse_rfc3339
+    parse_rfc3339,
+    calculate_price,
 )
