@@ -10,14 +10,14 @@ from .exception import ArlulaSessionError
 name = "arlulacore"
 
 # User agent setting
-sdk_version = "2.0.3"
+sdk_version = "3.1.0"
 py_version = sys.version.split(' ')[0]
 os_version = platform.platform()
 def_ua = "core-sdk " + \
     sdk_version + " python " + py_version + " OS " + os_version
 
 # Expected API version
-x_api_version = '2021-09'
+x_api_version = '2023-01'
 
 class Session:
     '''
@@ -28,7 +28,8 @@ class Session:
                  key: str,
                  secret: str,
                  user_agent: typing.Optional[str] = def_ua,
-                 url: typing.Optional[str] = "https://api.arlula.com"
+                 url: typing.Optional[str] = "https://api.arlula.com",
+                 test: typing.Optional[bool] = True,
                  ):
         # Encode the key and secret
         def atob(x): return x.encode('utf-8')
@@ -39,8 +40,11 @@ class Session:
             'User-Agent': user_agent,
             'X-API-Version': x_api_version
         }
+        if url is None:
+            url = "https://api.arlula.com"
         self.baseURL = url
-        self.validate_creds()
+        if test:
+            self.validate_creds()
 
     # Check the credentials are valid
     def validate_creds(self):
